<?php

namespace Drupal\webform_number_limit\Plugin\WebformHandler;

use Drupal\webform\Plugin\WebformElement\Number;

/**
 * Provides a trait for number limit webform handlers.
 */
trait NumberLimitWebformHandlerTrait {

  /**
   * Get key/value array of webform number elements.
   *
   * @return array
   *   A key/value array of webform number elements.
   */
  protected function getNumberElements(): array {
    $webform = $this->getWebform();
    $elements = $webform->getElementsInitializedAndFlattened();

    $options = [];
    foreach ($elements as $element) {
      $webform_element = $this->elementManager->getElementInstance($element);

      if ($webform_element instanceof Number || (isset($element['#options']) && !$this->hasStringKeys($element['#options']))) {
        $webform_key = $element['#webform_key'];
        $t_args = [
          '@title' => $webform_element->getAdminLabel($element),
          '@type' => $webform_element->getPluginLabel(),
        ];
        $options[$webform_key] = $this->t('@title (@type)', $t_args);
      }
    }

    // Exclude elements being used by other number limit handlers.
    $handlers = $webform->getHandlers();
    foreach ($handlers as $handler) {
      if (!$handler instanceof NumberLimitWebformHandler && !$handler instanceof OptionsNumberLimitWebformHandler) {
        continue;
      }
      if ($handler->getHandlerId() === $this->getHandlerId()) {
        continue;
      }

      $element_key = $handler->getSetting('element_key');
      unset($options[$element_key]);
    }

    return $options;
  }

  /**
   * Determine if an array has non-numeric keys.
   */
  protected function hasStringKeys(array $array): bool {
    $string_keys = array_filter(array_keys($array), function ($key) {
      return !is_numeric($key);
    });

    return count($string_keys) > 0;
  }

  /**
   * Show a message when the previously selected element is invalid.
   */
  protected function showMissingElementMessage(array &$form): void {
    $form['missing_element_message'] = [
      '#type' => 'webform_message',
      '#message_message' => $this->t("The previously selected element does not exist or isn't a number element anymore. Choose a different element."),
      '#message_type' => 'warning',
      '#weight' => -100,
    ];
  }

}
